open Util ;;    
open CrawlerServices ;;
open Order ;;
open Pagerank ;;


(* Uses matrix algebra to compute page ranks. Check pagerank.ml
 * to see the full implementation *)
module MoogleRanker
     = EigenvalueRanker (PageGraph) (PageScore) (struct 
       let do_random_jumps = Some 0.20
     end)

  

(* Dictionaries mapping words (strings) to sets of crawler links *)
module WordDict = Dict.Make(
  struct 
    type key = string
    type value = LinkSet.set
    let compare = string_compare
    let string_of_key = (fun s -> s)
    let string_of_value = LinkSet.string_of_set

    (* These functions are for testing purposes *)
    let gen_key () = ""
    let gen_key_gt x () = gen_key ()
    let gen_key_lt x () = gen_key ()
    let gen_key_random () = gen_key ()
    let gen_key_between x y () = None
    let gen_value () = LinkSet.empty
    let gen_pair () = (gen_key(),gen_value())
    let arbitrary_key () = QCheck.Gen.map (fun (x : key) -> x) QCheck.Gen.(small_string)
    let arbitrary_value () = QCheck.Gen.return LinkSet.empty
  end)

(* A query module that uses LinkSet and WordDict *)
module Q = Query.Query(
  struct
    module S = LinkSet
    module D = WordDict
  end)

let print s = 
  let _ = Printf.printf "%s\n" s in
  flush_all();;


(***********************************************************************)
(*    PART 1: CRAWLER                                                  *)
(***********************************************************************)

(* TODO: Build an index as follows:
 * 
 * Remove a link from the frontier (the set of links that have yet to
 * be visited), visit this link, add its outgoing links to the
 * frontier, and update the index so that all words on this page are
 * mapped to linksets containing this url.
 *
 * You can get the words on a page using the get_page function, which has
 * the form link -> page option. The page data type has fields for the
 * links and words of a page, called links and words.
 * 
 * Keep crawling until we've
 * reached the maximum number of links (n) or the frontier is empty.
 * Do not count pages that are not found by get_page towards the maximum
 * number of links. *)
let rec crawl (n:int) (frontier: LinkSet.set)
    (visited : LinkSet.set) (d:WordDict.dict) : WordDict.dict = 
  WordDict.empty
;;

let crawler () = 
  if num_pages_to_search == -1 then (Printf.printf
  "usage: ./moogle.native <port> <num-pages> <root-url>\n" ;
  exit 1)
  else
  crawl num_pages_to_search (LinkSet.singleton initial_link) LinkSet.empty
    WordDict.empty
;;

(* Debugging note: if you set debug=true in moogle.ml, it will print out your
 * index after crawling. *)
