open Dict ;;
(******************************************************************)
(* Run AVL dict tests.                                                 *)
(******************************************************************)

(* Create a dictionary mapping ints to strings using our 
 * AVLDict functor and run the tests.
 * 
 * Uncomment out the lines below when you are ready to test your
 * 2-3 tree implementation. *)

print_string "Running AVL Dict Tests\n";;

module IntStringAVLDict = Dict.AVLDict(Dict.IntStringDictArg) ;;
IntStringAVLDict.run_tests();;

print_string "\nAVL Dict Tests Complete\n";;

(******************************************************************)
(* Tests for Crawl.ml tests. Uncomment these to test your crawler *)
(******************************************************************)
(*
print_string "\nRunning Crawl Tests\n";;

(* Check if the links in a set of links are what they are expected to be. *)
let checkEntry (dict) (value: string) (quiet: bool) (values : string list) =
  match Crawl.WordDict.lookup dict value with
  | Some x -> let (lst, ans) = (Pagerank.LinkSet.fold (fun d x ->
      match x with
      | (a::b, true) ->
        if String.equal a (Pagerank.LinkSet.string_of_elt d) then (b, true) else ([], false)
      | _ -> ([], false)) (values, true) x) in
    let _ = if (ans == false || List.length lst > 0) && quiet == false then 
     (print_string ("Student Result: "^(Pagerank.LinkSet.string_of_set x));
      print_string ("\nCorrect Result: "^(List.fold_left (fun d x -> d^"; "^x) "" values))) in
    ans
  | _ -> if quiet == false then print_string ("Word "^value^" not found in dict\n") else (); false;;

(*Printing Function*)
let runTest (mes:string) (test:bool) =
  if test == false then (
    print_string mes;
    print_string " FAILED\n")
  else ();;

(* Crawler Tests on simple files. These tests run the crawler on the dummy files in crawlTests *)
(* and compare the result for some keyword to a hard coded list of files that keyword appears in. *)

let crawl_test_nolinks = Crawl.crawl 1
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/nolinks.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test no links" (checkEntry crawl_test_nolinks "NOLINKS" false ["file://crawltests/nolinks.html"]);;

let crawl_test_onelink = Crawl.crawl 2
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/onelink.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test one link 1" (checkEntry crawl_test_onelink "ONELINK" false ["file://crawltests/onelink.html"]);
runTest "test one link 2" (checkEntry crawl_test_onelink "NOLINKS" false ["file://crawltests/nolinks.html"]);;

let crawl_test_onelink_stop = Crawl.crawl 1
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/onelink.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test one link stop 1"
  (checkEntry crawl_test_onelink_stop "ONELINK" false ["file://crawltests/onelink.html"]);
runTest "test one link stop 2"
  ((checkEntry crawl_test_onelink_stop "NOLINKS" true ["file://crawltests/nolinks.html"]) == false);;

let crawl_test_twolinks = Crawl.crawl 5
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/twolinks.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test two links 1"
  (checkEntry crawl_test_twolinks "ONELINK" false
     ["file://crawltests/onelink.html"; "file://crawltests/onelink2.html"]);
runTest "test two links 2" (checkEntry crawl_test_twolinks "NOLINKS" false ["file://crawltests/nolinks.html"]);
runTest "test two links 3" (checkEntry crawl_test_twolinks "TWOLINKS" false ["file://crawltests/twolinks.html"]);;

let crawl_test_twolinks_stop = Crawl.crawl 2
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/twolinks.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test two links 1"
  (checkEntry crawl_test_twolinks_stop "ONELINK" false
     ["file://crawltests/onelink.html"]);
runTest "test two links 2" ((checkEntry crawl_test_twolinks_stop "NOLINKS" true ["file://crawltests/nolinks.html"]) == false);
runTest "test two links 3" (checkEntry crawl_test_twolinks_stop "TWOLINKS" false ["file://crawltests/twolinks.html"]);;

let crawl_test_badlink = Crawl.crawl 3
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/badlink.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test bad link 1"
  (checkEntry crawl_test_badlink "ONELINK" false
     ["file://crawltests/onelink.html"]);
runTest "test bad link 2" (checkEntry crawl_test_badlink "NOLINKS" false ["file://crawltests/nolinks.html"]);
runTest "test bad link 3" (checkEntry crawl_test_badlink "BADLINK" false ["file://crawltests/badlink.html"]);;

let crawl_test_circle = Crawl.crawl 100
    (Pagerank.LinkSet.singleton ({host=""; port=80; path="crawltests/circle1.html"}))
    Pagerank.LinkSet.empty
    Crawl.WordDict.empty in
runTest "test circle 1"
  (checkEntry crawl_test_circle "CIRCLE" false
     ["file://crawltests/circle1.html"; "file://crawltests/circle2.html";"file://crawltests/circle3.html" ]);;

print_string "\nCrawl Tests Complete\n";;
*)
